@@ -47,6 +47,7 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 #include <wlan/wlan_compat.h>
 #include <wlan/p80211types.h>
 #include <wlan/p80211meta.h>
@@ -58,6 +59,8 @@ int main(int argc, char **argv)
     int     c, g, i, totgrps, totitems, type;
     char    name[82];
 
+    int pad_id = 0;
+
 /* traverse the message metadata to build the structures and typedefs */
     for ( c = 1; c < msg_catlist_size; c++ )
     {
@@ -65,7 +68,6 @@ int main(int argc, char **argv)
         for ( g = 1; g < totgrps; g++ )
         {
 	    strcpy(name, "p80211msg");
-            printf("__WLAN_PRAGMA_PACK1__\n");
             printf("typedef struct %s_%s\n{\n", name,
                 msg_catlist[c].grplist[g].name);
             printf("\tUINT32\t\tmsgcode\t__WLAN_ATTRIB_PACK__;\n");
@@ -74,32 +76,81 @@ int main(int argc, char **argv)
             totitems = GETMETASIZE(msg_catlist[c].grplist[g].itemlist);
             for ( i = 1; i < totitems; i++ )
             {
-                type = P80211DID_TYPE(
-                            msg_catlist[c].grplist[g].itemlist[i].did);
+                type = p80211item_gettype(&msg_catlist[c].grplist[g].itemlist[i]);
 		switch ( type )
 		{
-                    case P80211_TYPE_BOUNDEDINT:
                     case P80211_TYPE_INT:
                     case P80211_TYPE_ENUMINT:
+                    case P80211_TYPE_BITARRAY:
+			    /* already aligned */
                          printf("\tp80211item_uint32_t");
                          printf("\t%s\t__WLAN_ATTRIB_PACK__;\n",
                              msg_catlist[c].grplist[g].itemlist[i].name);
 	                break;
 		    case P80211_TYPE_UNKDATA:
-		        printf("\tp80211item_unk%ld_t",
+			    /* already aligned */
+		        printf("\tp80211item_unk%d_t",
                             msg_catlist[c].grplist[g].itemlist[i].maxlen);
                         printf("\t%s\t__WLAN_ATTRIB_PACK__;\n",
                             msg_catlist[c].grplist[g].itemlist[i].name);
 			break;
-	            default:
-                         printf("\tp80211item_pstr%ld_t",
+		    case P80211_TYPE_INTARRAY:
+			    /* already aligned */
+		        printf("\tstruct {\n");
+		        printf("\t\tUINT32\tdid\t__WLAN_ATTRIB_PACK__;\n");
+		        printf("\t\tUINT16\tstatus\t__WLAN_ATTRIB_PACK__;\n");
+		        printf("\t\tUINT16\tlen\t__WLAN_ATTRIB_PACK__;\n");
+                        printf("\t\tUINT32\tdata[%d]\t__WLAN_ATTRIB_PACK__;\n",
+                            msg_catlist[c].grplist[g].itemlist[i].maxlen);
+		        printf("\t\t} %s\t__WLAN_ATTRIB_PACK__;\n",
+                            msg_catlist[c].grplist[g].itemlist[i].name);
+			break;
+		    case P80211_TYPE_MACARRAY:
+			    /* May not be aligned, thanks to variable-length array. */
+		        printf("\tstruct {\n");
+		        printf("\t\tUINT32\tdid\t__WLAN_ATTRIB_PACK__;\n");
+		        printf("\t\tUINT16\tstatus\t__WLAN_ATTRIB_PACK__;\n");
+		        printf("\t\tUINT16\tlen\t__WLAN_ATTRIB_PACK__;\n");
+		        printf("\t\tUINT32\tcnt\t__WLAN_ATTRIB_PACK__;\n");
+                        printf("\t\tUINT8\tdata[%d][WLAN_ADDR_LEN]\t__WLAN_ATTRIB_PACK__;\n",
+                            msg_catlist[c].grplist[g].itemlist[i].maxlen);
+
+			if (msg_catlist[c].grplist[g].itemlist[i].maxlen % 4)
+				printf("\t\tUINT8\tpad_%dB[%d]\t__WLAN_ATTRIB_PACK__;\n",
+				       pad_id++,
+				       (4 - ((msg_catlist[c].grplist[g].itemlist[i].maxlen * 6) % 4))); 
+			
+		        printf("\t\t} %s\t__WLAN_ATTRIB_PACK__;\n",
+                            msg_catlist[c].grplist[g].itemlist[i].name);
+			break;
+	            case P80211_TYPE_OCTETSTR:
+			    /* May not be aligned.  it's a string. */
+                         printf("\tp80211item_pstr%d_t",
                             msg_catlist[c].grplist[g].itemlist[i].maxlen);
                          printf("\t%s\t__WLAN_ATTRIB_PACK__;\n",
                              msg_catlist[c].grplist[g].itemlist[i].name);
+
+			 if ((msg_catlist[c].grplist[g].itemlist[i].maxlen + 1) % 4)
+				 printf("\tUINT8\tpad_%dC[%d]\t__WLAN_ATTRIB_PACK__;\n",
+					pad_id++,
+					(4 - ((msg_catlist[c].grplist[g].itemlist[i].maxlen + 1) % 4))); 
+
+	                break;
+	            default:
+			    /* May not be aligned.  it's a string. */
+                         printf("\tp80211item_pstr%d_t",
+                            msg_catlist[c].grplist[g].itemlist[i].maxlen);
+                         printf("\t%s\t__WLAN_ATTRIB_PACK__;\n",
+                             msg_catlist[c].grplist[g].itemlist[i].name);
+
+			 if ((msg_catlist[c].grplist[g].itemlist[i].maxlen + 1) % 4)
+				 printf("\tUINT8\tpad_%dD[%d]\t__WLAN_ATTRIB_PACK__;\n",
+					pad_id++,
+					(4 - ((msg_catlist[c].grplist[g].itemlist[i].maxlen + 1) % 4))); 
 	                break;
 		}
             }
-            printf("} %s_%s_t;\n__WLAN_PRAGMA_PACKDFLT__\n\n\n", 
+            printf("} __WLAN_ATTRIB_PACK__ %s_%s_t;\n\n", 
 	    	name,
                 msg_catlist[c].grplist[g].name);
         }
